@@ -4,13 +4,13 @@ use 5.010001;
 use Mouse::Role;
 use experimental 'smartmatch';
 
-# ABSTRACT: A Simple base module to implement almost every RESTful API with just a few lines of configuration
+# ABSTRACT: Web::API - A Simple base module to implement almost every RESTful API with just a few lines of configuration
 
-our $VERSION = '2.4.1'; # VERSION
+our $VERSION = '2.1'; # VERSION
 
 use LWP::UserAgent;
-use HTTP::Cookies 6.04;
-use Data::Printer colored => 1;
+use HTTP::Cookies;
+use Data::Dump 'dump';
 use XML::Simple;
 use URI::Escape::XS qw/uri_escape uri_unescape/;
 use JSON;
@@ -60,16 +60,12 @@ has 'api_key_field' => (
 );
 
 
-has 'api_version' => (
+has 'mapping' => (
     is      => 'rw',
-    isa     => 'Int',
-    default => sub { 1 },
+    default => sub { {} },
 );
 
 
-has 'mapping' => (is => 'rw');
-
-
 has 'wrapper' => (
     is      => 'rw',
     clearer => 'clear_wrapper',
@@ -90,13 +86,6 @@ has 'auth_type' => (
 );
 
 
-has 'auth_header' => (
-    is      => 'rw',
-    isa     => 'Str',
-    default => sub { 'Authorization' },
-);
-
-
 has 'default_method' => (
     is      => 'rw',
     isa     => 'Str',
@@ -119,18 +108,19 @@ has 'user_agent' => (
 
 
 has 'timeout' => (
-    is      => 'rw',
-    isa     => 'Int',
-    default => sub { 30 },
-    lazy    => 1,
+    is       => 'rw',
+    isa      => 'Int',
+    default  => sub { 30 },
+    required => 1,
 );
 
 
 has 'strict_ssl' => (
-    is      => 'rw',
-    isa     => 'Bool',
-    default => sub { 1 },
-    lazy    => 1,
+    is       => 'rw',
+    isa      => 'Bool',
+    default  => sub { 0 },
+    lazy     => 1,
+    required => 1,
 );
 
 
@@ -145,13 +135,7 @@ has 'agent' => (
 
 has 'retry_http_codes' => (
     is  => 'rw',
-    isa => 'ArrayRef[Int]',
-);
-
-
-has 'retry_errors' => (
-    is  => 'rw',
-    isa => 'ArrayRef[RegexpRef]',
+    isa => 'ArrayRef',
 );
 
 
@@ -252,12 +236,6 @@ has 'oauth_post_body' => (
     lazy    => 1,
 );
 
-
-has 'error_keys' => (
-    is  => 'rw',
-    isa => 'ArrayRef[Str]',
-);
-
 has 'json' => (
     is      => 'rw',
     isa     => 'JSON',
@@ -285,18 +263,6 @@ has 'xml' => (
     },
 );
 
-has '_decoded_response' => (
-    is      => 'rw',
-    isa     => 'Ref',
-    clearer => 'clear_decoded_response',
-);
-
-has '_response' => (
-    is      => 'rw',
-    isa     => 'HTTP::Response',
-    clearer => 'clear_response',
-);
-
 sub _build_agent {
     my ($self) = @_;
 
@@ -317,7 +283,7 @@ sub nonce {
 
 sub log {    ## no critic (ProhibitBuiltinHomonyms)
     my ($self, $msg) = @_;
-    print STDERR caller() . ': ' . $msg . $/;
+    print STDERR __PACKAGE__ . ': ' . $msg . $/;
     return;
 }
 
@@ -325,8 +291,6 @@ sub log {    ## no critic (ProhibitBuiltinHomonyms)
 sub decode {
     my ($self, $content, $content_type) = @_;
 
-    $self->log("decoding response from '$content_type'") if $self->debug;
-
     my $data;
     eval {
         if ($self->has_decoder) {
@@ -335,10 +299,7 @@ sub decode {
         }
         else {
             given ($content_type) {
-                when (/plain/) {
-                    chomp $content;
-                    $data = { text => $content };
-                }
+                when (/plain/) { $data = $content; }
                 when (/urlencoded/) {
                     foreach (split(/&/, $content)) {
                         my ($key, $value) = split(/=/, $_);
@@ -352,12 +313,10 @@ sub decode {
             }
         }
     };
-
-    die("couldn't decode payload using $content_type: $@\n" . np($content))
+    return { error => "couldn't decode payload using $content_type: $@\n"
+            . dump($content) }
         if ($@ || ref \$content ne 'SCALAR');
 
-    $self->_decoded_response($data);
-
     return $data;
 }
 
@@ -365,8 +324,6 @@ sub decode {
 sub encode {
     my ($self, $options, $content_type) = @_;
 
-    $self->log("encoding response to '$content_type'") if $self->debug;
-
     my $payload;
     eval {
         # custom encoder should only be run if called by Web::API otherwise we
@@ -389,7 +346,8 @@ sub encode {
             }
         }
     };
-    die("couldn't encode payload using $content_type: $@\n" . np(%$options))
+    return { error => "couldn't encode payload using $content_type: $@\n"
+            . dump($options) }
         if ($@ || ref \$payload ne 'SCALAR');
 
     return $payload;
@@ -397,7 +355,7 @@ sub encode {
 
 
 sub talk {
-    my ($self, $command, $uri, $options, $query_keys, $content_type) = @_;
+    my ($self, $command, $uri, $options, $content_type) = @_;
 
     my $method = uc($command->{method} || $self->default_method);
     my $oauth_req;
@@ -405,10 +363,6 @@ sub talk {
     # handle different auth_types
     given (lc $self->auth_type) {
         when ('basic') { $uri->userinfo($self->user . ':' . $self->api_key); }
-        when ('header') {
-            $self->header->{ $self->auth_header } =
-                "Token token=" . $self->api_key;
-        }
         when ('hash_key') {
             my $api_key_field = $self->api_key_field;
             if ($self->mapping and not $command->{no_mapping}) {
@@ -443,7 +397,7 @@ sub talk {
                 and (($self->oauth_post_body and $method eq 'POST')
                     or $method ne 'POST'))
             {
-                $opts{extra_params} = { %$options, %{ $query_keys || {} } };
+                $opts{extra_params} = $options;
             }
 
             $oauth_req = Net::OAuth->request("protected resource")->new(%opts);
@@ -470,25 +424,30 @@ sub talk {
         }
         else {
             $payload = $self->encode($options, $content_type->{out});
+
+            # got an error while encoding? return it
+            return $payload
+                if (ref $payload eq 'HASH' && exists $payload->{error});
+
             $self->log("send payload: $payload") if $self->debug;
         }
     }
 
-    # append query keys to URI
-    $uri->query_param_append($_ => $query_keys->{$_}) for keys %$query_keys;
-
     $uri = $oauth_req->to_url if ($self->auth_type eq 'oauth_params');
 
     # build headers
-    my %header = %{ $self->header };
+    my %header;
     if (exists $command->{headers} and ref $command->{headers} eq 'HASH') {
-        %header = (%header, %{ $command->{headers} });
+        %header = (%{ $self->header }, %{ $command->{headers} });
     }
-    my $headers = HTTP::Headers->new("Accept" => $content_type->{in}, %header);
+    else {
+        %header = %{ $self->header };
+    }
+    my $headers = HTTP::Headers->new(%header, "Accept" => $content_type->{in});
 
     if ($self->debug) {
         $self->log("uri: $method $uri");
-        $self->log("extra headers: " . np(%header)) if (%header);
+        $self->log("extra headers: " . dump(\%header)) if (%header);
         $self->log("OAuth headers: " . $oauth_req->to_authorization_header)
             if ($self->auth_type eq 'oauth_header');
     }
@@ -517,12 +476,44 @@ sub talk {
     $self->agent->cookie_jar($self->cookies);
 
     # do the actual work
-    return $self->request($request);
+    my $response = $self->request($request);
+    return $response if ref $response eq 'HASH' and exists $response->{error};
+
+    $self->log("recv payload: " . $response->decoded_content)
+        if $self->debug;
+
+    # collect response headers
+    my $response_headers;
+    $response_headers->{$_} = $response->header($_)
+        foreach ($response->header_field_names);
+
+    my $answer = {
+        header  => $response_headers,
+        code    => $response->code,
+        content => $self->decode(
+            $response->decoded_content,
+            ($response_headers->{'Content-Type'} || $content_type->{in})
+        ),
+        raw => $response->content,
+    };
+
+    unless ($response->is_success || $response->is_redirect) {
+        $self->log("error: "
+                . $response->status_line
+                . $/
+                . "message: "
+                . $response->decoded_content)
+            if $self->debug;
+
+        $answer->{error} = "request failed: " . $response->status_line;
+    }
+
+    return $answer;
 }
 
 
 sub map_options {
-    my ($self, $options, $command) = @_;
+    my ($self, $options, $command, $content_type) = @_;
 
     my %opts;
 
@@ -530,9 +521,9 @@ sub map_options {
     %opts = %{ $command->{default_attributes} }
         if exists $command->{default_attributes};
 
-    # then map everything in $options, overwriting default_attributes if necessary
+    # then map everything in $options, overwriting detault_attributes if necessary
     if ($self->mapping and not $command->{no_mapping}) {
-        $self->log("mapping hash:\n" . np(%{ $self->mapping })) if $self->debug;
+        $self->log("mapping hash:\n" . dump($self->mapping)) if $self->debug;
 
         # do the key and value mapping of options hash and overwrite defaults
         foreach my $key (keys %$options) {
@@ -551,45 +542,38 @@ sub map_options {
         $options = { %opts, %$options };
     }
 
-    return $options;
-}
+    # then check existence of mandatory attributes
+    if ($command->{mandatory}) {
+        $self->log("mandatory keys:\n" . dump(\@{ $command->{mandatory} }))
+            if $self->debug;
 
+        my @missing_attrs;
+        foreach my $attr (@{ $command->{mandatory} }) {
+            my @bits = split /\./, $attr;
+            my $node = $options;
 
-sub check_mandatory {
-    my ($self, $options, $mandatory) = @_;
+            push(@missing_attrs, $attr)
+                unless @bits == grep {
+                       ref $node eq "HASH"
+                    && exists $node->{$_}
+                    && ($node = $node->{$_} // {})
+                } @bits;
+        }
 
-    $self->log("mandatory keys:\n" . np(@$mandatory))
-        if $self->debug;
-
-    my @missing_attrs;
-    foreach my $attr (@$mandatory) {
-        push(@missing_attrs, $attr)
-            unless $self->key_exists($attr, $options);
+        return { error => 'mandatory attributes for this command missing: '
+                . join(', ', @missing_attrs) }
+            if @missing_attrs;
     }
 
-    die 'mandatory attributes for this command missing: '
-        . join(', ', @missing_attrs)
-        . $/
-        if @missing_attrs;
+    # wrap all options in wrapper key(s) if requested
+    my $method = uc($command->{method} || $self->default_method);
+    $options =
+        wrap($options, $command->{wrapper} || $self->wrapper, $content_type)
+        unless ($method =~ m/^(GET|HEAD|DELETE)$/);
 
-    return;
-}
+    $self->log("options:\n" . dump($options)) if $self->debug;
 
-
-sub key_exists {
-    my ($self, $path, $hash) = @_;
-
-    my @bits = split /\./, $path;
-    my $node = $hash;
-
-    return $node
-        if @bits == grep {
-               ref $node eq "HASH"
-            && exists $node->{$_}
-            && ($node = $node->{$_} // {})
-        } @bits;
-
-    return;
+    return $options;
 }
 
 
@@ -601,10 +585,10 @@ sub wrap {
         # XML needs wrapping into extra array ref layer to make XML::Simple
         # behave correctly
         if ($content_type =~ m/xml/) {
-            $options = { $_ => [$options] } for (reverse @$wrapper);
+            $options = { $_ => [$options] } for (reverse @{$wrapper});
         }
         else {
-            $options = { $_ => $options } for (reverse @$wrapper);
+            $options = { $_ => $options } for (reverse @{$wrapper});
         }
     }
     elsif (defined $wrapper) {
@@ -619,11 +603,9 @@ sub request {
     my ($self, $request) = @_;
 
     my $response;
-    my $error;
+    my $err;
 
-    if (   ($self->retry_http_codes and scalar(@{ $self->retry_http_codes }))
-        or ($self->retry_errors and scalar(@{ $self->retry_errors })))
-    {
+    if ($self->retry_http_codes) {
         my $times = $self->retry_times;
         my $n     = 0;
 
@@ -636,18 +618,15 @@ sub request {
                 if $self->debug;
 
             $response = eval { $self->agent->request($request) };
-            $error = $@;
-
-            # if the user agent died there was a connection issue, definitely retry those
-            unless ($error) {
-                $self->_response($response);
-
-                $self->log("recv payload: " . $response->decoded_content)
+            my $err = $@;
+            if ($err) {
+                $response->{error} = $err;
+            }
+            else {
+                $self->log("response code was: " . $response->code)
                     if $self->debug;
-
                 return $response
-                    unless $self->needs_retry($response,
-                    $request->header('Accept'));
+                    unless $response->code ~~ $self->retry_http_codes;
             }
 
             sleep $self->retry_delay if $times;    # Do not sleep in last time
@@ -655,241 +634,98 @@ sub request {
     }
     else {
         $response = eval { $self->agent->request($request) };
-        $error = $@;
-
-        $self->log("recv payload: " . $response->decoded_content)
-            if $response and $self->debug;
+        $response->{error} = $@ if $@;
     }
 
-    $self->_response($response);
-
-    die $error if $error;
-
     return $response;
 }
 
 
-sub needs_retry {
-    my ($self, $response, $content_type) = @_;
-
-    $self->log("response code was: " . $response->code)
-        if $self->debug;
-
-    return 1 if $response->code ~~ $self->retry_http_codes;
-
-    if (    $self->retry_errors
-        and scalar(@{ $self->retry_errors })
-        and $self->error_keys
-        and scalar(@{ $self->error_keys }))
-    {
-        # we need to decode the response content to be able to find a custom
-        # error field
-        my $content = $self->decode($response->decoded_content,
-            ($response->header('Content-Type') || $content_type));
-
-        my $error = $self->find_error($content);
-
-        return unless $error;
-
-        return 1 if map { $error =~ $_ } @{ $self->retry_errors };
-    }
-
-    return;
-}
-
-
-sub find_error {
-    my ($self, $content) = @_;
-
-    for (@{ $self->error_keys || [] }) {
-        $self->log("checking for error at ($_)") if $self->debug;
-
-        my $node = $self->key_exists($_, $content);
-
-        if ($node) {
-            $self->log("found error: '$node' at ($_)") if $self->debug;
-            return $node;
-        }
-    }
-
-    return;
-}
-
-
-sub format_response {
-    my ($self, $response, $ct, $error) = @_;
-
-    my $answer;
-
-    if ($response) {
-
-        # collect response headers
-        my $response_headers;
-        $response_headers->{$_} = $response->header($_)
-            foreach ($response->header_field_names);
-
-        # decode content if necessary
-        unless ($self->_decoded_response) {
-            if (    defined $response->decoded_content
-                and length($response->decoded_content) > 0
-                and $response->decoded_content =~ m/\S/)
-            {
-                $self->_decoded_response(
-                    eval {
-                        $self->decode($response->decoded_content,
-                            ($response_headers->{'Content-Type'} || $ct));
-                    });
-                $error ||= $@;
-            }
-        }
-
-        # search for and expose errors
-        $error ||= $self->find_error($self->_decoded_response);
-
-        $answer = {
-            header  => $response_headers,
-            code    => $response->code,
-            content => $self->_decoded_response,
-            raw     => $response->content,
-            cookies => $self->cookies->get_cookies($self->base_url),
-        };
-
-        unless ($response->is_success || $response->is_redirect) {
-            $error ||= $response->status_line;
-        }
-    }
-
-    if ($error) {
-        chomp($error);
-        $self->log("ERROR: $error");
-        $answer->{error} = $error;
-    }
-
-    return $answer;
-}
-
-
-sub build_uri {
-    my ($self, $command, $options, $path) = @_;
-
-    my $uri = URI->new($self->base_url);
-    my $p   = $uri->path;
-
-    if ($path) {
-        $p .= '/' . $path;
-
-        # parse all mandatory ID keys from URI path
-        # format: /path/with/some/:id/and/:another_id/fun.js
-        my @mandatory = ($path =~ m/:(\w+)/g);
-
-        # and replace placeholders
-        foreach my $key (@mandatory) {
-            die "required {$key} option missing\n"
-                unless (exists $options->{$key});
-
-            my $encoded_option = uri_escape(delete $options->{$key});
-            $p =~ s/:$key/$encoded_option/gex;
-        }
-    }
-    else {
-        $p .= "/$command";
-    }
-
-    $p .= '.' . $self->extension if ($self->extension);
-    $uri->path($p);
-
-    return $uri;
-}
-
-
-sub build_content_type {
-    my ($self, $command) = @_;
-
-    return {
-        in => $command->{incoming_content_type}
-            || $command->{content_type}
-            || $CONTENT_TYPE{ $self->extension }
-            || $self->incoming_content_type
-            || $self->content_type,
-        out => $command->{outgoing_content_type}
-            || $command->{content_type}
-            || $self->outgoing_content_type
-            || $self->content_type,
-    };
-}
-
-
-sub DESTROY { }
-
-
 sub AUTOLOAD {
     my ($self, %options) = @_;
 
-    $self->clear_decoded_response;
-    $self->clear_response;
-
-    # sanity checks
-    die "Attribute (base_url) is required\n" unless $self->base_url;
-    if ($self->auth_type =~ m/^oauth_/) {
-        for (qw(consumer_secret access_token access_secret)) {
-            die "Attribute ($_) is required\n" unless $self->$_;
-        }
-    }
-
     my ($command) = $AUTOLOAD =~ /([^:]+)$/;
 
-    my $ct;
-    my $response = eval {
-        die "unknown command: $command\n"
-            unless (exists $self->commands->{$command});
+    return { error => "unknown command: $command" }
+        unless (exists $self->commands->{$command});
 
-        my $options = \%options;
+    my $options = \%options;
 
-        # construct URI
-        my $uri =
-            $self->build_uri($command, $options,
-            $self->commands->{$command}->{path});
+    # construct URI path
+    my $uri  = URI->new($self->base_url);
+    my $path = $uri->path;
 
-        # first select the right content types for encoding/decoding
-        $ct = $self->build_content_type($self->commands->{$command});
+    # keep for backward compatibility
+    if ($self->commands->{$command}->{require_id}) {
+        return { error => "required {id} attribute missing" }
+            unless (exists $options->{id});
+        my $id = delete $options->{id};
+        $path .= '/' . $self->commands->{$command}->{pre_id_path}
+            if (exists $self->commands->{$command}->{pre_id_path});
+        $path .= '/' . $id;
+        $path .= '/' . $self->commands->{$command}->{post_id_path}
+            if (exists $self->commands->{$command}->{post_id_path});
+    }
+    elsif (exists $self->commands->{$command}->{path}) {
+        $path .= '/' . $self->commands->{$command}->{path};
 
-        # then map options if necessary
-        $options =
-            $self->map_options($options, $self->commands->{$command}, $ct->{in})
-            if (((keys %$options) and ($ct->{out} =~ m/(xml|json|urlencoded)/))
-            or (exists $self->commands->{$command}->{default_attributes})
-            or (exists $self->commands->{$command}->{mandatory}));
+        # parse all mandatory ID keys from URI path
+        # format: /path/with/some/:id/and/:another_id/fun.js
+        my @mandatory = ($self->commands->{$command}->{path} =~ m/:(\w+)/g);
 
-        # then check existence of mandatory attributes
-        $self->check_mandatory($options,
-            $self->commands->{$command}->{mandatory})
-            if exists $self->commands->{$command}->{mandatory};
+        # and replace placeholders
+        foreach my $key (@mandatory) {
+            return { error => "required {$key} attribute missing" }
+                unless exists $options->{$key};
 
-        # then extract query keys from options to prevent them being wrapped
-        # in the next step
-        my $query_keys;
-        foreach my $key (keys %$options) {
-            $query_keys->{$key} = delete $options->{$key}
-                if $key ~~ $self->commands->{$command}->{query_keys};
+            my $encoded_option = uri_escape(delete $options->{$key});
+            $path =~ s/:$key/$encoded_option/gex;
         }
+    }
+    else {
+        $path .= "/$command";
+    }
 
-        # finally wrap all options in wrapper key(s) if requested
-        my $method =
-            uc($self->commands->{$command}->{method} || $self->default_method);
-        $options =
-            wrap($options,
-            $self->commands->{$command}->{wrapper} || $self->wrapper,
-            $ct->{in})
-            unless ($method =~ m/^(GET|HEAD|DELETE)$/);
+    $path .= '.' . $self->extension if ($self->extension);
+    $uri->path($path);
 
-        $self->log("options:\n" . np(%$options)) if $self->debug;
+    # configure in/out content types
+    # order of precedence should be:
+    # command based incoming_content_type
+    # command based general content_type
+    # content type based on extension (only for incoming)
+    # global incoming_content_type
+    # global general content_type
+    my $content_type;
+    $content_type->{in} =
+           $self->commands->{$command}->{incoming_content_type}
+        || $self->commands->{$command}->{content_type}
+        || $CONTENT_TYPE{ $self->extension }
+        || $self->incoming_content_type
+        || $self->content_type;
+    $content_type->{out} =
+           $self->commands->{$command}->{outgoing_content_type}
+        || $self->commands->{$command}->{content_type}
+        || $self->outgoing_content_type
+        || $self->content_type;
 
-        # do the talking
-        return $self->talk($self->commands->{$command},
-            $uri, $options, $query_keys, $ct);
-    };
+    # manage options
+    $options = $self->map_options($options, $self->commands->{$command},
+        $content_type->{in})
+        if ((
+                (keys %$options)
+            and ($content_type->{out} =~ m/(xml|json|urlencoded)/))
+        or (exists $self->commands->{$command}->{default_attributes})
+        or (exists $self->commands->{$command}->{mandatory}));
+    return $options if (exists $options->{error});
 
-    return $self->format_response($self->_response, $ct->{in}, $@);
+    # do the call
+    my $response =
+        $self->talk($self->commands->{$command}, $uri, $options, $content_type);
+
+    $self->log("response:\n" . dump($response)) if $self->debug;
+
+    return $response;
 }
 
 
@@ -903,23 +739,19 @@ __END__
 
 =head1 NAME
 
-Web::API - A Simple base module to implement almost every RESTful API with just a few lines of configuration
+Web::API - Web::API - A Simple base module to implement almost every RESTful API with just a few lines of configuration
 
 =head1 VERSION
 
-version 2.4.1
+version 2.1
 
 =head1 SYNOPSIS
 
-B<NOTE:> as of version 2.1 C<strict_ssl> is enabled by default for obvious security
-reasons, this may break your current library implementation, sorry.
-
 Implement the RESTful API of your choice in 10 minutes, roughly.
 
     package Net::CloudProvider;
 
-    use Mouse;
-    
+    use Any::Moose;
     with 'Web::API';
 
     our $VERSION = "0.1";
@@ -1006,7 +838,7 @@ later use as:
 
     use Net::CloudProvider;
     
-    my $nc = Net::CloudProvider->new(user => 'foobar', api_key => 'secret');
+    my $nc = Net::CloudProvider(user => 'foobar', api_key => 'secret');
     my $response = $nc->create_node({
         id                             => 'funnybox',
         hostname                       => 'node.funnybox.com',
@@ -1030,89 +862,63 @@ module you are writing.
 the following keys are valid/possible:
 
     method
+    require_id
     path
-    mandatory
+    pre_id_path
+    post_id_path
+    wrapper
     default_attributes
-    headers
+    mandatory
     extension
     content_type
     incoming_content_type
     outgoing_content_type
-    wrapper
-    query_keys
-    require_id (deprecated, use path)
-    pre_id_path (deprecated, use path)
-    post_id_path (deprecated, use path)
 
-the request path for commands is being build as:
+the request path for non require_id commands is being build as:
 
     $base_url/$path.$extension
 
-an example for C<path>:
+accordingly requests with require_id:
 
-    path => 'users/:user_id/labels'
+    $base_url/$pre_id_path/$id/$post_id_path.$extension
 
-this will add C<user_id> to the list of mandatory keys for this command
-automatically.
+whereas $id can be any arbitrary object like a domain, that the API in question
+does operations on.
 
 =head2 base_url (required)
 
 get/set base URL to API, can include paths
 
-=head2 api_key (required in most cases)
+=head2 api_key (required)
 
-get/set API key (also used as basic auth password)
+get/set api_key
 
 =head2 user (optional)
 
-get/set API username/account name
+get/set username/account name
 
 =head2 api_key_field (optional)
 
-get/set name of the hash key that has to hold the C<api_key>
-e.g. in POST content payloads
-
-=head2 api_version (optional)
-
-get/set API version to be used
-
-default: 1
+get/set name of the hash key in the POST data structure that has to hold the api_key
 
 =head2 mapping (optional)
 
-supply mapping table, hashref of format { "key" => "value", ... }
+supply mapping table, hashref of format { key => value }
+
+default: undef
 
 =head2 wrapper (optional)
 
-get/set name of the key that is used to wrap all options of a command in.
-unfortunately some APIs increase the depth of a hash by wrapping everything into
-a single key (who knows why...), which means this:
-
-    $wa->command(%options);
-
-turns C<%options> into:
-
-    { wrapper => \%options }
-
-before encoding and sending it off.
-
 =head2 header (optional)
 
-get/set custom headers sent with every request
+get/set custom headers sent with each request
 
 =head2 auth_type
 
-get/set authentication type. currently supported are only 'basic', 'header',
-'hash_key', 'get_params', 'oauth_header', 'oauth_params' or 'none'
+get/set authentication type. currently supported are only 'basic', 'hash_key', 'get_params', 'oauth_header', 'oauth_params' or 'none'
 
 default: none
 
-=head2 auth_header (optional)
-
-get/set the name of the header used for Authorization credentials
-
-default: Authorization
-
 =head2 default_method (optional)
 
 get/set default HTTP method
@@ -1131,32 +937,25 @@ default: "Web::API $VERSION"
 
 =head2 timeout (optional)
 
-get/set L<LWP::UserAgent> timeout
+get/set LWP::UserAgent timeout
 
 =head2 strict_ssl (optional)
 
-enable/disable strict SSL certificate hostname checking as a convenience
-alternatively you can supply your own LWP::Useragent compatible agent for
-the C<agent> attribute.
+enable/disable strict SSL certificate hostname checking
 
-default: true
+default: false
 
 =head2 agent (optional)
 
-get/set L<LWP::UserAgent> object
+get/set LWP::UserAgent object
 
 =head2 retry_http_codes (optional)
 
 get/set array of HTTP response codes that trigger a retry of the request
 
-=head2 retry_errors (optional)
-
-define an array reference of regexes that should trigger a retry of the request
-if matched against an error found via one of the C<error_keys>
-
 =head2 retry_times (optional)
 
-get/set number of times a request will be retried at most
+get/set amount of times a request will be retried at most
 
 default: 3
 
@@ -1168,11 +967,6 @@ default: 1.0
 
 =head2 content_type (optional)
 
-global content type, which is used for in and out going request/response
-headers and to encode and decode the payload if no other more specific content
-types are set, e.g. C<incoming_content_type>, C<outgoing_content_type> or
-content types set individually per command attribute.
-
 default: 'text/plain'
 
 =head2 incoming_content_type (optional)
@@ -1185,18 +979,11 @@ default: undef
 
 =head2 debug (optional)
 
-enable/disabled debug logging
-
-default: false
+default: 0
 
 =head2 cookies (optional)
 
-this is used to store and retrieve cookies before and after requests were made
-to keep authenticated sessions alive for the time this object exists in memory
-you can add your own cookies to be send with every request. See
-L<HTTP::Cookies> for more information.
-
-default: HTTP::Cookies->new()
+default: HTTP::Cookies->new
 
 =head2 consumer_secret (required for all oauth_* auth_types)
 
@@ -1216,40 +1003,20 @@ default: undef
 
 =head2 encoder (custom options encoding subroutine)
 
-Receives C<\%options> and C<content-type> as the only 2 arguments and has to
-return a single scalar.
+Receives options and content-type as the only 2 arguments
 
 default: undef
 
 =head2 decoder (custom response content decoding subroutine)
 
-Receives C<content> and C<content-type> as the only 2 scalar arguments and has
-to return a single hash reference.
+Receives content and content-type as the only 2 arguments
 
 default: undef
 
 =head2 oauth_post_body (required for all oauth_* auth_types)
 
-enable/disable adding of command options as extra parameters to the OAuth
-request generation and therefor be included in the OAuth signature calculation.
-
 default: true
 
-=head2 error_keys
-
-get/set list of array keys that will be search for in the decoded response data
-structure. the same format as for mandatory keys is supported:
-
-    some.deeply.nested.error.message
-
-will search for an error message at
-
-    $decoded_response->{some}->{deeply}->{nested}->{error}->{messsage}
-
-and if the key exists and its value is defined it will be provided as
-C<$response->{error}> and matched against all regexes from the `retry_errors`
-array ref if provided to trigger a retry on particular errors.
-
 =head1 INTERNAL SUBROUTINES/METHODS
 
 =head2 nonce
@@ -1266,60 +1033,14 @@ generates new OAuth nonce for every request
 
 =head2 map_options
 
-=head2 check_mandatory
-
-=head2 key_exists
-
 =head2 wrap
 
 =head2 request
 
 retry request with delay if C<retry_http_codes> is set, otherwise just try once.
 
-=head2 needs_retry
-
-returns true if the HTTP code or error found match either C<retry_http_codes>
-or C<retry_errors> respectively.
-returns false otherwise.
-
-if C<retry_errors> are defined it will try to decode the response content and
-store the decoded structure internally so we don't have to decode again at the
-end.
-
-needs the last response object and the 'Accept' content type header from the
-request for decoding.
-
-=head2 find_error
-
-go through C<error_keys> and find a potential error message in the decoded/parsed
-response and return it.
-
-=head2 format_response
-
-=head2 build_uri
-
-=head2 build_content_type
-
-configure in/out content types
-
-order of precedence:
-1. per command C<incoming_content_type> / C<outgoing_content_type>
-2. per command general C<content_type>
-3. content type based on file path extension (only for incoming)
-4. global C<incoming_content_type> / C<outgoing_content_type>
-5. global general C<content_type>
-
-=head2 DESTROY
-
-catch DESTROY call and tear down / clean up if necessary
-at this point there is nothing to do though. This prevents
-AUTOLOAD from logging an unknown command error message
-
 =head2 AUTOLOAD magic
 
-install a method for each new command and call it in an C<eval {}> to catch
-exceptions and set an error in a unified way.
-
 =head1 BUGS
 
 Please report any bugs or feature requests on GitHub's issue tracker L<https://github.com/nupfel/Web-API/issues>.
@@ -1327,7 +1048,7 @@ Pull requests welcome.
 
 =head1 SUPPORT
 
-You can find documentation for this module with the L<perldoc(1)> command.
+You can find documentation for this module with the perldoc command.
 
     perldoc Web::API
 
@@ -1353,10 +1074,6 @@ L<http://cpanratings.perl.org/d/Web::API>
 
 =back
 
-=head1 SEE ALSO
-
-L<HTTP::Cookies>, L<LWP::UserAgent>, L<Net::OAuth>
-
 =head1 AUTHOR
 
 Tobias Kirschstein <lev@cpan.org>
